@@ -30,11 +30,19 @@ import os
 import glob
 import sys
 import argparse
+import collections
+import warnings
 
 import pyalpm
 
-class InvalidSyntax(Exception):
-	pass
+class InvalidSyntax(Warning):
+	def __init__(self, filename, problem, arg):
+		self.filename = filename
+		self.problem = problem
+		self.arg = arg
+
+	def __str__(self):
+		return "unable to parse %s, %s: %r" % (self.filename, self.problem, self.arg)
 
 # Options that may occur several times in a section. Their values should be
 # accumulated in a list.
@@ -52,10 +60,12 @@ LIST_OPTIONS = (
 SINGLE_OPTIONS = (
 	'RootDir',
 	'DBPath',
+	'GPGDir',
 	'LogFile',
 	'Architecture',
 	'XferCommand',
-	'CleanMethod'
+	'CleanMethod',
+	'VerifySig'
 )
 
 BOOLEAN_OPTIONS = (
@@ -63,7 +73,8 @@ BOOLEAN_OPTIONS = (
 	'ShowSize',
 	'UseDelta',
 	'TotalDownload',
-	'CheckSpace'
+	'CheckSpace',
+	'VerbosePkgLists'
 )
 
 def pacman_conf_enumerator(path):
@@ -98,6 +109,8 @@ def pacman_conf_enumerator(path):
 			# repos only have the Server option
 			if key == 'Server' and equal == '=':
 				yield (current_section, 'Server', value)
+			elif key == 'VerifySig' and equal == '=':
+				yield (current_section, 'VerifySig', value)
 			else:
 				raise InvalidSyntax(f.name, 'invalid key for repository configuration', line)
 			continue
@@ -108,20 +121,36 @@ def pacman_conf_enumerator(path):
 			elif key in SINGLE_OPTIONS:
 				yield (current_section, key, value)
 			else:
-				print(InvalidSyntax(f.name, 'unrecognized option', key))
+				warnings.warn(InvalidSyntax(f.name, 'unrecognized option', key))
 		else:
 			if key in BOOLEAN_OPTIONS:
 				yield (current_section, key, True)
 			else:
-				print(InvalidSyntax(f.name, 'unrecognized option', key))
+				warnings.warn(InvalidSyntax(f.name, 'unrecognized option', key))
+
+_logmask = pyalpm.LOG_ERROR | pyalpm.LOG_WARNING
+
+def cb_log(level, line):
+	if not (level & _logmask):
+		return
+	if level & pyalpm.LOG_ERROR:
+		line = "ERROR: " + line
+	elif level & pyalpm.LOG_WARNING:
+		line = "WARNING: " + line
+	elif level & pyalpm.LOG_DEBUG:
+		line = "DEBUG: " + line
+	elif level & pyalpm.LOG_FUNCTION:
+		line = "FUNC: " + line
+	sys.stderr.write(line)
 
 class PacmanConfig(object):
 	def __init__(self, conf = None, options = None):
 		self.options = {}
-		self.repos = {}
+		self.repos = collections.OrderedDict()
 		self.options["RootDir"] = "/"
-		self.options["DBPath"] = "/var/lib/pacman"
-		self.options["LogFile"] = "/var/lib/pacman"
+		self.options["DBPath"]  = "/var/lib/pacman"
+		self.options["GPGDir"]  = "/etc/pacman.d/gnupg/"
+		self.options["LogFile"] = "/var/log/pacman.log"
 		self.options["Architecture"] = os.uname()[-1]
 		if conf is not None:
 			self.load_from_file(conf)
@@ -139,32 +168,49 @@ class PacmanConfig(object):
 					self.options[key] = value
 			else:
 				servers = self.repos.setdefault(section, [])
-				assert(key == 'Server')
-				servers.append(value)
+				if key == 'Server':
+					servers.append(value)
+		if "CacheDir" not in self.options:
+			self.options["CacheDir"]= ["/var/cache/pacman/pkg"]
 
 	def load_from_options(self, options):
+		global _logmask
 		if options.root is not None:
 			self.options["RootDir"] = options.root
 		if options.dbpath is not None:
 			self.options["DBPath"] = options.dbpath
+		if options.gpgdir is not None:
+			self.options["GPGDir"] = options.gpgdir
 		if options.arch is not None:
 			self.options["Architecture"] = options.arch
 		if options.logfile is not None:
 			self.options["LogFile"] = options.logfile
+		if options.cachedir is not None:
+			self.options["CacheDir"] = [option.cachedir]
+		if options.debug:
+			_logmask = 0xffff
 
-	def apply(self):
-		pyalpm.options.root = self.options["RootDir"]
-		pyalpm.options.dbpath = self.options["DBPath"]
-		pyalpm.options.arch = self.options["Architecture"]
-		pyalpm.options.logfile = self.options["LogFile"]
+	def apply(self, h):
+		h.arch = self.options["Architecture"]
+		h.logfile = self.options["LogFile"]
+		h.gpgdir = self.options["GPGDir"]
+		h.cachedirs = self.options["CacheDir"]
+		h.logcb = cb_log
 
 		# set sync databases
 		for repo, servers in self.repos.items():
-			db = pyalpm.register_syncdb(repo)
+			db = h.register_syncdb(repo, 0)
+			db_servers = []
 			for rawurl in servers:
 				url = rawurl.replace("$repo", repo)
 				url = url.replace("$arch", self.options["Architecture"])
-				db.url = url
+				db_servers.append(url)
+			db.servers = db_servers
+
+	def initialize_alpm(self):
+		h = pyalpm.Handle(self.options["RootDir"], self.options["DBPath"])
+		self.apply(h)
+		return h
 
 	def __str__(self):
 		return("PacmanConfig(options=%s, repos=%s)" % (str(self.options), str(self.repos)))
@@ -187,20 +233,27 @@ def make_parser(*args, **kwargs):
 	common.add_argument('--config', metavar = '<file>',
 			action = 'store', dest = 'config', type = str,
 			help = 'set an alternate configuration file')
+	common.add_argument('--debug',
+			action='store_true', dest='debug',
+			help='display debug messages')
 	common.add_argument('--logfile', metavar = '<file>',
 			action = 'store', dest = 'logfile', type = str,
 			help = 'set an alternate log file')
+	common.add_argument('--gpgdir', metavar = '<dir>',
+			action = 'store', dest = 'gpgdir', type = str,
+			help = 'set an alternate log file')
+	common.add_argument('--cachedir', metavar = '<dir>',
+			action = 'store', dest = 'cachedir', type = str,
+			help = 'set an alternate cche location')
 	return parser
 
 def init_with_config(configpath):
 	"Reads configuration from given path and apply it to libalpm"
-	pyalpm.initialize()
 	config = PacmanConfig(conf = configpath)
-	config.apply()
+	return config.initialize_alpm()
 
 def init_with_config_and_options(options):
 	"Reads configuration from file and commandline options, and apply it to libalpm"
-	pyalpm.initialize()
 	# read config file
 	if options.config is not None:
 		config_file = options.config
@@ -208,6 +261,6 @@ def init_with_config_and_options(options):
 		config_file = "/etc/pacman.conf"
 
 	conf = PacmanConfig(conf = config_file, options = options)
-	conf.apply()
+	return conf.initialize_alpm()
 
 # vim: set ts=4 sw=4 tw=0 noet:
