@@ -1,5 +1,5 @@
 /**
- * package.c : wrapper class around pmpkg_t
+ * package.c : wrapper class around alpm_pkg_t
  *
  *  Copyright (c) 2011 Rémy Oudompheng <remy@archlinux.org>
  *
@@ -25,16 +25,24 @@
 
 #include <Python.h>
 
-extern PyTypeObject AlpmPackageType;
+typedef struct _AlpmPackage {
+  PyObject_HEAD
+  alpm_pkg_t *c_data;
+  int needs_free;
+} AlpmPackage;
+
+#define ALPM_PACKAGE(self) (((AlpmPackage*)(self))->c_data)
+
+PyTypeObject AlpmPackageType;
 int PyAlpmPkg_Check(PyObject *object);
 
 void pyalpm_pkg_unref(PyObject *object);
 
 PyObject *pyalpm_package_from_pmpkg(void* data);
-pmpkg_t *pmpkg_from_pyalpm_pkg(PyObject *object);
+alpm_pkg_t *pmpkg_from_pyalpm_pkg(PyObject *object);
 
 int pylist_pkg_to_alpmlist(PyObject *list, alpm_list_t **result);
 
-PyObject *pyalpm_package_load(PyObject *self, PyObject *args);
+PyObject *pyalpm_package_load(PyObject *self, PyObject *args, PyObject *kwargs);
 
-#endif // _PYALPM_PACKAGE_H
+#endif
